@@ -1,44 +1,14 @@
 """access.py
 
-Access control
-
-  Definitions:
-
-  - principal: a user or role
-  - membership: parent resource type for privacy sharing
-  - model: database model name (e.g. Person)
-  - resource: resource type (e.g. person)
-  - rbac: role-based access control (defined in rbac.yaml)
-  - role: a group name (e.g. admin or list-<id>-<level>)
-  - privacy: sharing options as defined in rbac.yaml (e.g. \
-    secret [default], public, invitee, member, manager)
-  - actions: crudlghij (create, read, update, del, list, guest/member, \
-    host/manager, invitee, join)
-
-  In rbac.yaml, define the RBAC policies for each principal/resource
-  combination. That file will be parsed into a singleton variable upon initial
-  startup.  This implementation implements RBAC similar to that of
-  kubernetes or AWS IAM, with the added capability of a simple privacy
-  permission within each object (database record) which creates an implied
-  ACL for read-only access by members of the object's group.
-
-  Group names currently used are:
-
-  - admin
-  - user
-  - pending
-  - person
-  - <resource>-<id>-<privacy>
-
-  These are defined in session_auth.py's account_login() method.
-
 created 20-may-2019 by richb@instantlinux.net
 refactored 6-mar-2020
 """
 
-from datetime import datetime
+from base64 import b64encode
+from datetime import datetime, timedelta
 from flask import g, request
 from flask_babel import _
+import hashlib
 import logging
 import re
 from sqlalchemy import or_
@@ -46,6 +16,7 @@ from sqlalchemy.orm.exc import NoResultFound
 import yaml
 
 from .service_config import ServiceConfig
+from .utils import gen_id
 
 LEVELS = {}
 POLICIES = {}
@@ -55,6 +26,36 @@ PRIV_RES = {}
 class AccessControl(object):
     """Role-based access control
 
+    Definitions:
+
+    - principal: a user or role
+    - membership: parent resource type for privacy sharing
+    - model: database model name (e.g. Person)
+    - resource: resource type (e.g. person)
+    - rbac: role-based access control (defined in rbac.yaml)
+    - role: a group name (e.g. admin or list-<id>-<level>)
+    - privacy: sharing options as defined in rbac.yaml (e.g. \
+      secret [default], public, invitee, member, manager)
+    - actions: crudlghij (create, read, update, del, list, guest/member, \
+      host/manager, invitee, join)
+
+    In rbac.yaml, define the RBAC policies for each principal/resource
+    combination. That file will be parsed into a singleton variable upon
+    initial startup.  This implementation implements RBAC similar to that of
+    kubernetes or AWS IAM, with the added capability of a simple privacy
+    permission within each object (database record) which creates an implied
+    ACL for read-only access by members of the object's group.
+
+    Group names currently used are:
+
+    - admin
+    - user
+    - pending
+    - person
+    - <resource>-<id>-<privacy>
+
+    These are defined in session_auth.py's account_login() method.
+
     Args:
       policy_file (str): name of the yaml definitions file
       model (obj): a model to be validated for permissions
@@ -64,15 +65,26 @@ class AccessControl(object):
             self.policies = POLICIES['policies']
             self.privacy_levels = LEVELS['levels']
             self.private_res = PRIV_RES['private_res']
-            creds = request.authorization
             self.model = model
             self.models = ServiceConfig().models
             self.resource = model.__name__.lower() if model else None
             self.auth = None
-            if creds:
+            header_auth = ServiceConfig().config.HEADER_AUTH_APIKEY
+            if header_auth in request.headers:
+                # TODO: add support for HMAC request signing in place
+                # of this plain-text X-Api-Key header method
+                prefix, secret = request.headers.get(header_auth).split('.')
+                item = g.session.get(None, self.apikey_hash(secret)[:8],
+                                     key_id=prefix)
+                if item:
+                    self.auth = item.get('auth').split(':')
+                    self.uid = uid = item.get('sub')
+            elif request.authorization:
+                uid = self.uid = request.authorization.username
+                secret = request.authorization.password
                 try:
                     self.auth = g.session.get(
-                        creds.username, creds.password, arg='auth').split(':')
+                        uid, secret, arg='auth').split(':')
                 except AttributeError:
                     pass
             self.primary_res = self.private_res[0]['resource']
@@ -82,13 +94,11 @@ class AccessControl(object):
                     ev = self._parse_id(role, self.primary_res)
                     if ev:
                         self.auth_ids[self.primary_res].append(ev)
-                uid = self.uid = creds.username
-                self.account_id = g.session.get(uid, creds.password, 'acc')
-                self.identity = g.session.get(uid, creds.password, 'identity')
+                self.account_id = g.session.get(uid, secret, 'acc')
+                self.identity = g.session.get(uid, secret, 'identity')
             else:
-                logging.warning('no-auth resource=%s attempted' %
-                                self.resource)
-                self.auth = self.uid = self.account_id = self.identity = None
+                # For anonymous-access paths that don't require security
+                self.uid = self.account_id = self.identity = None
 
     def load_rbac(self, filename):
         """ Read RBAC default policies from rbac.yaml, process any
@@ -201,7 +211,6 @@ class AccessControl(object):
         Returns:
           set: actions available to principal
         """
-
         if query:
             try:
                 record = query.one()
@@ -235,7 +244,8 @@ class AccessControl(object):
 
         actions, defaults = (set(), set())
         if privacy == 'public' or (
-                id and '%s-%s-%s' % (membership, id, privacy) in self.auth):
+                self.auth and id and '%s-%s-%s' % (membership, id, privacy)
+                in self.auth):
             defaults = set('r')
 
         for policy in self.policies[self.resource]:
@@ -273,6 +283,63 @@ class AccessControl(object):
         actions -= deny_delete
         return actions if len(actions) else defaults
 
+    def apikey_create(self):
+        """Generate an API key - a 41-byte string. First 8 characters (48
+        bits) are an access key ID prefix; last 32 characters (192
+        bits) are the secret key.
+
+
+        Returns: tuple
+          key ID (str) - public portion of key
+          secret (str) - secret portion
+          hashvalue (str) - hash value for database
+        """
+        secret = gen_id(length=35, prefix='')[-32:]
+        key_id = gen_id(prefix='')
+        return key_id, secret, self.apikey_hash(secret)
+
+    def apikey_verify(self, key_id, secret):
+        """Verify an API key
+
+        Args:
+          key_id (str): the public key_id at time of generation
+          secret (str): the unhashed secret
+
+        Returns: tuple
+          uid (str): User ID if valid
+          scopes (list): list of scope IDs
+        """
+        try:
+            record = g.db.query(self.models.APIkey).filter_by(
+                prefix=key_id, hashvalue=self.apikey_hash(secret),
+                status='active').one()
+            if not record.last_used or (
+                    record.last_used + timedelta(hours=6) < datetime.utcnow()
+                    and record.expires < datetime.utcnow()):
+                record.last_used = datetime.utcnow()
+                g.db.commit()
+        except NoResultFound:
+            logging.info(dict(action='api_key', key_id=key_id,
+                              message='not found'))
+            return None, None
+        except Exception as ex:
+            logging.error(dict(action='api_key', message=str(ex)))
+            return None, None
+        if record.expires and record.expires < datetime.now():
+            logging.info(dict(action='api_key', key_id=key_id,
+                              uid=record.uid, message='expired'))
+            return None, None
+        return record.uid, record.scopes
+
+    @staticmethod
+    def apikey_hash(secret):
+        """Generate a hash value from the secret
+        Args:
+          secret (str): secret key
+        """
+        return b64encode(
+            hashlib.sha256(secret.encode()).digest()).decode('utf8')
+
     @staticmethod
     def _parse_id(role, resource):
         """Given role <res>-<id>-<level>
