@@ -1,11 +1,7 @@
 """session_auth
 
-Session Authorization
-  Functions for login, password and role authorization
-
 created 26-mar-2020 by richb@instantlinux.net
 """
-
 from datetime import datetime, timedelta
 from flask import g, request
 from flask_babel import _
@@ -24,12 +20,16 @@ from .const import Constants
 from .messaging.confirmation import Confirmation
 from .service_config import ServiceConfig
 from .service_registry import ServiceRegistry
-from .utils import gen_id
+from .utils import gen_id, utcnow
+
+APIKEYS = {}
 
 
 class SessionAuth(object):
     """Session Authorization
 
+    Functions for login, password and role authorization
+
     Args:
       func_send(function): name of function for sending message
     """
@@ -123,7 +123,7 @@ class SessionAuth(object):
         duration = (self.login_admin_limit if account.is_admin
                     else self.login_session_limit)
         ses = g.session.create(account.uid, roles, acc=account.id,
-                               identity=account.owner.identity,  ttl=duration)
+                               identity=account.owner.identity, ttl=duration)
         if ses:
             retval = dict(
                 jwt_token=jwt.encode(
@@ -136,6 +136,61 @@ class SessionAuth(object):
         else:
             return dict(message=_(u'rejected')), 403
 
+    def api_access(self, apikey, roles_from=None):
+        """ Access using API key
+
+        Args:
+          apikey (str): the API key
+          roles_from (obj): model for which to look up authorizations
+
+        Returns:
+          dict: uid, scopes (None if not authorized)
+        """
+        global APIKEYS
+        acc = AccessControl()
+        key_id, secret = apikey.split('.')
+        if key_id not in APIKEYS or utcnow() > APIKEYS[key_id]['expires']:
+            # Note - local caching in APIKEYS global var reduces database
+            # queries on Accounts table
+            uid, scopes = acc.apikey_verify(key_id, secret)
+            if not uid:
+                return None
+            try:
+                account = g.db.query(self.models.Account).filter_by(
+                    uid=uid, status='active').one()
+            except NoResultFound:
+                logging.info(dict(action='api_key', uid=uid,
+                                  message='account not active'))
+                return None
+            except Exception as ex:
+                logging.error(dict(action='api_key', message=str(ex)))
+                return None
+            APIKEYS[key_id] = dict(
+                account_id=account.id,
+                expires=utcnow() + timedelta(seconds=self.config.REDIS_TTL),
+                identity=account.owner.identity,
+                roles=['admin', 'user'] if account.is_admin else ['user'],
+                scopes=[item.name for item in scopes], uid=uid)
+            if roles_from:
+                APIKEYS[key_id]['roles'] += self.get_roles(uid, roles_from)
+        item = APIKEYS[key_id]
+        uid = item['uid']
+        logging.debug(dict(action='api_key', uid=uid, scopes=item['scopes']))
+        duration = (self.login_admin_limit if 'admin' in item['roles']
+                    else self.login_session_limit)
+        token = acc.apikey_hash(secret)[:8]
+        ses = None
+        try:
+            # Look for existing session in redis cache
+            ses = g.session.get(
+                uid, token, arg='auth', key_id=key_id).split(':')
+        except AttributeError:
+            ses = g.session.create(uid, item['roles'], acc=item['account_id'],
+                                   identity=item['identity'],
+                                   ttl=duration, key_id=key_id, nonce=token)
+        if ses:
+            return dict(uid=uid, scopes=item['scopes'])
+
     def register(self, identity, username, name, template='confirm_new'):
         """Register a new account: create related records in database
         and send confirmation token to new user
@@ -399,15 +454,28 @@ def basic(username, password, required_scopes=None):
         if session:
             g.session.delete(username, password)
         return None
-    return {'uid': username}
+    return dict(uid=username)
 
 
-def api_key(token, required_scopes=None):
-    """ API key authentication - not yet implemented
+def api_key(apikey, required_scopes=None):
+    """ API key authentication
 
     Args:
-      token (str): the token
+      apikey (str): the key
       required_scopes (list): permissions requested
+
+    Returns:
+      dict: uid if successful (None otherwise)
     """
-    logging.info('action=api_key token=%s' % token)
-    return {'sub': 'user1', 'scope': ''}
+    models = ServiceConfig().models
+    if len(apikey) > 96 or '.' not in apikey:
+        return None
+    retval = SessionAuth().api_access(apikey, roles_from=models.List)
+    if not retval:
+        return retval
+    if required_scopes and (set(required_scopes) & set(retval['scopes']) !=
+                            set(required_scopes)):
+        logging.info(dict(action='api_key', prefix=apikey.split('.')[0],
+                          scopes=retval['scopes'], message='denied'))
+        return None
+    return retval
