@@ -1,22 +1,17 @@
 """session_manager.py
 
-Session Manager
-  Each login session is stored as an encrypted JSON dict in redis, indexed
-  by sub:token
-
 created 8-may-2019 by richb@instantlinux.net
 """
 
 from datetime import datetime, timedelta
 import json
 import logging
-import random
 import redis
-import string
 import time
 
 from .aes_encrypt import AESEncrypt
 from .service_config import ServiceConfig
+from .utils import gen_id
 
 saved_redis = None
 
@@ -24,6 +19,9 @@ saved_redis = None
 class SessionManager(object):
     """Session Manager - for active user sessions
 
+    Each login session is stored as an encrypted JSON dict in redis, indexed
+    by sub:token
+
     Args:
       ttl (int): seconds until a session expires
       redis_conn (obj): connection to redis service
@@ -40,7 +38,7 @@ class SessionManager(object):
         self.ttl = ttl or self.config.REDIS_TTL
         self.aes = AESEncrypt(self.config.REDIS_AES_SECRET)
 
-    def create(self, uid, roles, **kwargs):
+    def create(self, uid, roles, key_id=None, **kwargs):
         """Create a session, which is an encrypted JSON object with the values
         defined in https://tools.ietf.org/html/rfc7519 for JWT claim names:
 
@@ -63,6 +61,7 @@ class SessionManager(object):
         Args:
           uid: User ID
           roles: Authorized roles
+          key_id: session key ID for redis (defaults to uid)
           nonce: a unique identifier for the token (random if not specified)
           ttl: duration of session (defaulted from class init)
         Returns:
@@ -71,7 +70,7 @@ class SessionManager(object):
             (as above), along with parameters passed into this function
         """
 
-        token = kwargs.pop('nonce', _gen_id(prefix=''))
+        token = kwargs.pop('nonce', gen_id(prefix=''))
         ttl = kwargs.pop('ttl', self.ttl)
         params = dict(
             auth=':'.join(roles),
@@ -79,23 +78,24 @@ class SessionManager(object):
                 seconds=ttl)).strftime('%s'),
             iss=self.config.JWT_ISSUER, jti=token, sub=uid
         )
-        key = 'uid:%s:%s' % (uid, token[-3:])
+        key = 'ses:%s:%s' % (key_id or uid, token[-3:])
         content = {**params, **kwargs}  # noqa
         if self.connection.set(key, self.aes.encrypt(json.dumps(
                 content)), ex=ttl, nx=True):
             return content
 
-    def get(self, uid, token, arg=None):
+    def get(self, uid, token, arg=None, key_id=None):
         """Get one or all key-value pairs stored by session create
 
         Args:
           uid (str): User ID
           token (str): The token value passed from create as 'jti'
           arg (str): key of desired value (None to fetch all)
+          key_id (str): session key ID for redis (defaults to uid)
         Returns:
           dict or str: single value or dictionary of all session keys
         """
-        key = 'uid:%s:%s' % (uid, token[-3:])
+        key = 'ses:%s:%s' % (key_id or uid, token[-3:])
         try:
             data = json.loads(self.aes.decrypt(
                 self.connection.get(key)))
@@ -112,7 +112,7 @@ class SessionManager(object):
             return data[arg] if arg in data else None
         return data
 
-    def update(self, uid, token, arg, value):
+    def update(self, uid, token, arg, value, key_id=None):
         """Update a specified session key
 
         Args:
@@ -120,29 +120,28 @@ class SessionManager(object):
           token (str): The token value passed from create as 'jti'
           arg (str): key to update
           value (str): new value for key
+          key_id (str): session key ID for redis (defaults to uid)
         """
-        key = 'uid:%s:%s' % (uid, token[-3:])
+        key = 'ses:%s:%s' % (key_id or uid, token[-3:])
         data = self.get(uid, token)
         data[arg] = value
         self.connection.set(key, self.aes.encrypt(
             json.dumps(data)), ex=self.ttl)
 
-    def delete(self, uid, token):
+    def delete(self, uid, token, key_id=None):
         """Cancel a session
 
         Args:
           uid: User ID
           token (str): The token value passed from create as 'jti'
+          key_id (str): session key ID for redis
         """
-        self.connection.delete('uid:%s:%s' % (uid, token[-3:]))
+        self.connection.delete('ses:%s:%s' % (key_id or uid, token[-3:]))
 
 
 class Mutex:
     """Simple mutex implementation for non-clustered Redis
 
-    Tried to implement this as inner class to DRY out the init,
-    but ... no joy.
-
     Args:
       lockname (str): a unique name for the lock
       redis_host (str): IP or DNS name of redis service
@@ -157,7 +156,7 @@ class Mutex:
             redis_conn or redis.Redis(host=redis_host,
                                       port=config.REDIS_PORT, db=0))
         self.ttl = ttl or config.REDIS_TTL
-        self.lock_signature = _gen_id()
+        self.lock_signature = gen_id()
         self.lockname = lockname
         self.maxwait = maxwait
 
@@ -187,17 +186,3 @@ class Mutex:
 
     def __exit__(self, exc_type, exc_value, exc_traceback):
         self.release()
-
-
-# TODO: resolve another DRY problem - this yields a circular dependency
-# when invoked as 'import lib.utils'
-def _gen_id(length=8, prefix='x-', chars=(
-        '-' + string.digits + string.ascii_uppercase + '_' +
-        string.ascii_lowercase)):
-    def _int2base(x, chars, base=64):
-        return _int2base(x // base, chars, base).lstrip(chars[0]) + chars[
-            x % base] if x else chars[0]
-    return (prefix +
-            _int2base((datetime.utcnow() - datetime(2018, 1, 1)).days * 8 +
-                      random.randint(0, 8), chars) +
-            ''.join(random.choice(chars) for i in range(length - 3)))
